__author__ = "hugo.inzirillo"

from typing import List

from bifrost import NapoleonPositionServiceConnector

from napoleontoolbox.models.filter import SignalComputationFilter


class NapoleonPositionService(object):
    def __init__(self):
        self.__position_connector = NapoleonPositionServiceConnector()

    @property
    def position_connector(self):
        return self.__position_connector

    def filter(self, filter: SignalComputationFilter) -> List[dict]:
        return self.position_connector.filter(data=filter)



if __name__ == '__main__':
    a = NapoleonPositionService()



import numpy as np
import scipy
import pandas as pd
from scipy.stats import linregress

from napoleontoolbox.signal import signal_utility

from sklearn import preprocessing
from sklearn.preprocessing import StandardScaler
from functools import partial
from numpy.lib.stride_tricks import as_strided as stride
import pandas as pd
from scipy.optimize import Bounds, LinearConstraint, minimize
from napoleontoolbox.utility import metrics
import numpy as np
from napoleontoolbox.signal import signal_utility
from napoleontoolbox.connector import napoleon_connector
import json

from napoleontoolbox.parallel_run import signal_result_analyzer
from sklearn.metrics import accuracy_score
from functools import partial


class SimpleThresholdModel():

    def __init__(self, lo=False, magnitude=True, transform_features=False, leading_correlations=5,  nb_selected=10, number_per_year=365):
        self.model = None
        self.seed = 0
        self.nb_selected = nb_selected
        self.number_per_year = number_per_year
        self.leading_correlations = leading_correlations
        self.transform_features = transform_features
        self.magnitude = magnitude
        self.lo = lo


    def calibrate(self, X, y):
        return

    def fit(self, X_train, y_train, X_val, y_val):
        print(f'finding the {self.nb_selected} most correlated features with the output')

        X = X_train.copy().fillna(0.)
        y = y_train.copy().fillna(0.)

        output_correlations = []
        leading_output_correlations = []

        for daily_feat in X_train.columns:
            correlation_matrix = np.corrcoef(X[daily_feat], y)
            output_correlations.append(
                {
                    'feature': daily_feat,
                    'correlation': correlation_matrix[0][1]
                }
            )
            leading_correlation_matrix = np.corrcoef(X.iloc[-self.leading_correlations:, X.columns.get_loc(daily_feat)], y.iloc[-self.leading_correlations:])
            leading_output_correlations.append(
                {
                    'feature': daily_feat,
                    'correlation': leading_correlation_matrix[0][1]
                }
            )
        output_correlations_df = pd.DataFrame(output_correlations)
        output_correlations_df['abs_correlation'] = abs(output_correlations_df['correlation'])
        output_correlations_df = output_correlations_df.sort_values(by='abs_correlation', ascending=False)
        output_correlations_df['effect_sign']=np.sign(output_correlations_df['correlation'])
        output_correlations_df.index = output_correlations_df.feature

        leading_output_correlations_df = pd.DataFrame(leading_output_correlations)
        leading_output_correlations_df['abs_correlation'] = abs(leading_output_correlations_df['correlation'])
        leading_output_correlations_df = leading_output_correlations_df.sort_values(by='abs_correlation', ascending=False)
        leading_output_correlations_df['effect_sign'] = np.sign(leading_output_correlations_df['correlation'])
        leading_output_correlations_df.index = leading_output_correlations_df.feature



        selected_output_correlations_df = output_correlations_df.head(self.nb_selected)


        self.selected_features = list(selected_output_correlations_df.feature)

        selected_leading_output_correlations_df = leading_output_correlations_df[leading_output_correlations_df.feature.apply(lambda x: x in self.selected_features) ]

        self.leading_correlation_effects = selected_leading_output_correlations_df['correlation'].to_dict()

        self.correlation_effects = selected_output_correlations_df['correlation'].to_dict()
        selected_X = X[self.selected_features].copy()
        if self.transform_features:
            scaler = StandardScaler()
            scaler.fit(selected_X)
            transformed_selected_X = scaler.transform(selected_X)
            transformed_selected_X_df = pd.DataFrame(data = transformed_selected_X, columns = selected_X.columns, index = selected_X.index)
            self.scaler = scaler
        else :
            transformed_selected_X_df = selected_X.copy()

        past_values_dic = {}
        for feature_to_investigate in self.selected_features:

            x = transformed_selected_X_df[feature_to_investigate].values.copy()
            past_values_dic[feature_to_investigate] = x[:-self.leading_correlations]
        self.past_values_dic = past_values_dic

        return

    def predict(self, X_test):
        X = X_test.copy()
        selected_X = X[self.selected_features].copy()
        if self.transform_features:
            transformed_selected_X = self.scaler.transform(selected_X)
            transformed_selected_X_df = pd.DataFrame(data = transformed_selected_X, columns = selected_X.columns, index = selected_X.index)
        else :
            transformed_selected_X_df = selected_X.copy()
        for feature_to_investigate in self.selected_features:
            #print(feature_to_investigate)
            whole_correlation = self.correlation_effects[feature_to_investigate]
            leading_past_values = self.past_values_dic[feature_to_investigate]
            leading_correlations = self.leading_correlation_effects[feature_to_investigate]

            # long correlation way
            lag_signal_way = np.sign(whole_correlation)
            lead_signal_way = np.sign(leading_correlations)

            x = transformed_selected_X_df[feature_to_investigate].values.copy()
            lead_lag_ar =  x.mean()/leading_past_values.mean()

            lead_lag_correlation_modifier = abs(leading_correlations/whole_correlation)

            # the two signals are in the same way
            # the whole lookback correlation matches the leading correlation
            if lead_signal_way == lag_signal_way:
                #x = x*lag_signal_way*lead_lag_correlation_coefficient
                if self.magnitude:
                    x = x*lag_signal_way*lead_lag_correlation_modifier
                else:
                    x = np.sign(x) * lag_signal_way * lead_lag_correlation_modifier

            else:
                ### the lead does not match the lag
                ### the lead does not match the lag
                ### we go contrarian
                if self.lo:
                    x[:] =  0.
                else:
                    if self.magnitude:
                        x = -x * lag_signal_way * lead_lag_correlation_modifier
                    else:
                        x = -np.sign(x) * lag_signal_way * lead_lag_correlation_modifier


            transformed_selected_X_df[feature_to_investigate]=x
        return transformed_selected_X_df.mean(axis = 1)

    def get_features_importance(self, features_names):
        return



import dash_html_components as html


def format_percent_value(value):
    return f'{round(value, 2)}%'

def generate_table(dataframe, max_rows=10):
    return html.Table([
        html.Thead(
            html.Tr([html.Th(col) for col in dataframe.columns])
        ),
        html.Tbody([
            html.Tr([
                html.Td(format_percent_value(dataframe.iloc[i][col])) for col in dataframe.columns
            ]) for i in range(min(len(dataframe), max_rows))
        ])
    ])


__author__ = "hugo.inzirillo"

import logging
import sys
from abc import ABCMeta, abstractmethod

__version__ = "0.0.1"


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances.keys():
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class AbstractLoggerManager(metaclass=ABCMeta):

    @abstractmethod
    def get_logger(self, name=None):
        """

        Parameters
        ----------
        name : current name of the npxlogger

        Returns
        -------

        """
        return NapoleonLoggerManager._loggers[name]

    @property
    @abstractmethod
    def basic_config(self):
        """

        Returns
        -------

        """
        raise NotImplemented

    @property
    @abstractmethod
    def config(self) -> dict:
        """

        Returns
        -------

        """
        raise NotImplemented

    @property
    @abstractmethod
    def handler(self):
        """

        Returns
        -------

        """
        raise NotImplemented

    @property
    @abstractmethod
    def log_level(self):
        """

        Returns
        -------

        """
        raise NotImplemented

    @property
    @abstractmethod
    def handler_level(self):
        """

        Returns
        -------

        """
        raise NotImplemented


class _LoggerManager(AbstractLoggerManager):

    @property
    def log_level(self):
        return logging.DEBUG

    @property
    def handler_level(self):
        return logging.NOTSET

    @property
    def handler(self) -> logging.StreamHandler:
        console = logging.StreamHandler(stream=sys.stdout)
        console.setLevel(self.handler_level)
        console.setFormatter(logging.Formatter(self.get_format()))
        return console

    def get_logger(self, name=None):
        """

        Parameters
        ----------
        name : current name of the npxlogger

        Returns
        -------

        """
        if name is None:
            logging.basicConfig()
            return logging.getLogger()

    @property
    def basic_config(self):
        """

        Returns
        -------

        """
        logging.basicConfig()
        raise logging.getLogger()

    @property
    def config(self) -> dict:
        """

        Returns
        -------

        """
        return dict(filename="napoleontoolbox.log",
                    format=self.get_format(),
                    level=self.log_level)

    def get_format(self):
        return "%(levelname) -10s %(asctime)s | module : %(module)s | line : %(lineno)s | method : %(funcName)s | %(message)s"


class _NapoleonLoggerManager(_LoggerManager):
    __metaclass__ = Singleton

    _loggers = {}

    def __init__(self, *args, **kwargs):
        super(_NapoleonLoggerManager, self).__init__()
        self.__config = None
        pass

    def get_logger(self, name=None) -> logging.Logger:
        """

        Parameters
        ----------
        name : current name of the npxlogger

        Returns
        -------

        """

        if name not in _NapoleonLoggerManager._loggers.keys():
            logging.basicConfig(**self.config)
            _NapoleonLoggerManager._loggers[name] = logging.getLogger(str(name))
            _NapoleonLoggerManager._loggers[name].addHandler(self.handler)
            return _NapoleonLoggerManager._loggers[name]

        elif not name:

            return super(_NapoleonLoggerManager, self).get_logger()


NapoleonLoggerManager = _NapoleonLoggerManager
log = _NapoleonLoggerManager().get_logger(__package__)



import pandas as pd
import numpy as np
import re

def add_datepart(df, fldname, drop=True, time=False):
    if fldname not in df.columns:
        df[fldname] = df.index
    "Helper function that adds columns relevant to a date."
    # add_datepart(weather, "Date", drop=False)
    # add_datepart(googletrend, "Date", drop=False)
    # add_datepart(train, "Date", drop=False)
    # add_datepart(test, "Date", drop=False)
    fld = df[fldname]
    fld_dtype = fld.dtype
    if isinstance(fld_dtype, pd.core.dtypes.dtypes.DatetimeTZDtype):
        fld_dtype = np.datetime64

    if not np.issubdtype(fld_dtype, np.datetime64):
        df[fldname] = fld = pd.to_datetime(fld, infer_datetime_format=True)
    targ_pre = re.sub('[Dd]ate$', '', fldname)
    attr = ['Year', 'Month', 'Week', 'Day', 'Hour', 'Dayofweek', 'Dayofyear',
            'Is_month_end', 'Is_month_start', 'Is_quarter_end', 'Is_quarter_start', 'Is_year_end', 'Is_year_start']
    if time: attr = attr + ['Hour', 'Minute', 'Second']
    for n in attr: df[targ_pre + n] = getattr(fld.dt, n.lower())
    if drop: df.drop(fldname, axis=1, inplace=True)


def add_rebalancing_datepart(df, fldname,  rebalancing_method = 'monthly', last = True):
    if fldname not in df.columns:
        df[fldname] = df.index
    "Helper function that adds columns relevant to a date."
    # add_datepart(weather, "Date", drop=False)
    # add_datepart(googletrend, "Date", drop=False)
    # add_datepart(train, "Date", drop=False)
    # add_datepart(test, "Date", drop=False)
    fld = df[fldname]
    fld_dtype = fld.dtype
    if isinstance(fld_dtype, pd.core.dtypes.dtypes.DatetimeTZDtype):
        fld_dtype = np.datetime64

    if not np.issubdtype(fld_dtype, np.datetime64):
        df[fldname] = fld = pd.to_datetime(fld, infer_datetime_format=True)
    targ_pre = re.sub('[Dd]ate$', '', fldname)
    if rebalancing_method == 'monthly':
        attr =  ['Month']
    elif rebalancing_method == 'weekly':
        attr =  ['Week']
    else:
        raise Exception('not supported rebalancing frequency')

    for n in attr: df[targ_pre + n] = getattr(fld.dt, n.lower())

    if rebalancing_method == 'monthly':
        if last:
            df['is_rebalancing'] = abs(df['Month'].diff().shift(-1).fillna(0.))>0
        else:
            df['is_rebalancing'] = abs(df['Month'].diff().fillna(0.))>0
        df.drop('Month', axis=1, inplace=True)

    elif rebalancing_method == 'weekly':
        if last:
            df['is_rebalancing'] = abs(df['Week'].diff().shift(-1).fillna(0.))>0
        else:
            df['is_rebalancing'] = abs(df['Week'].diff().fillna(0.))>0
        df.drop('Week', axis=1, inplace=True)
    else:
        raise Exception('not supported rebalancing frequency')
    df.drop(fldname, axis=1, inplace=True)




#!/usr/bin/env python
# coding: utf-8

from abc import ABC, abstractmethod

import pandas as pd
import numpy as np

from napoleontoolbox.file_saver import dropbox_file_saver
from napoleontoolbox.signal import signal_generator
from napoleontoolbox.connector import napoleon_connector
from napoleontoolbox.signal import signal_utility
from napoleontoolbox.parallel_run import signal_result_analyzer
from napoleontoolbox.parallel_run import launcher_utility
import json

from pathlib import Path

def daily_bc_apply_standard_mean_discretization(signals_df=None, s=21, method ='find_best_three_states_discretization_lo'):
    day_signal_columns = [me_sig for me_sig in signals_df.columns if 'signal' in me_sig ]
    signals_np = signals_df[day_signal_columns].values  # [:, len(self.generic_columns):]
    full_y_true = signals_df['close'].pct_change().fillna(0.)
    y_pred_discretized = np.zeros(full_y_true.shape)
    for row_index in range(s, signals_np.shape[0]):
        starting_train = row_index - s
        stopping_train = row_index
        prediction = row_index
        rolling_np = signals_np[starting_train:stopping_train, :]
        y_train_true = full_y_true[starting_train:stopping_train]
        y_train_pred = np.mean(rolling_np, axis=1)

        if method == 'find_best_three_states_discretization':
            lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization(
                y_train_pred, y_train_true)
            y_next_pred = np.nanmean(signals_np[row_index, :])
            y_next_pred_discrete = signal_utility.apply_discretization_three_states(y_next_pred, upper_threshold,
                                                                                    lower_threshold)
        elif method == 'find_best_three_states_discretization_lo':
            lower_quantile_threshold, upper_quantile_threshold, lower_threshold, upper_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_three_states_discretization_lo(
                y_train_pred, y_train_true)
            y_next_pred = np.nanmean(signals_np[row_index, :])
            y_next_pred_discrete = signal_utility.apply_discretization_three_states_lo(y_next_pred, upper_threshold,
                                                                                       lower_threshold)
        elif method == 'find_best_two_states_discretization':
            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
                y_train_pred, y_train_true)

            y_next_pred = np.nanmean(signals_np[row_index, :])
            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
        elif method == 'find_best_two_states_discretization_lo':
            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization_lo(
                y_train_pred, y_train_true)
            y_next_pred = np.nanmean(signals_np[row_index, :])
            y_next_pred_discrete = signal_utility.apply_discretization_two_states_lo(y_next_pred, splitting_threshold)

        y_pred_discretized[prediction] = y_next_pred_discrete
    return y_pred_discretized

class AbstractRunner(ABC):
    def __init__(self, calibration_starting_date = None,  calibration_running_date = None, starting_date = None,  running_date = None,  drop_token=None, dropbox_backup = True, underlying = None, frequence='daily', selected_algo='',algo='' ,freqly_return_pkl_filename_suffix='freqly_candels.pkl',aggregated_pkl_file_suffix='freqly_to_mix.pkl', aggregated_pkl_mapping_file_suffix = 'freqly_to_mix_mapping.pkl', list_pkl_file_suffix = 'my_list.pkl', local_root_directory='../data/', user = 'napoleon', add_turn_over = False, skipping_size=200, recompute_all = True, db_path_suffix=None, daily_ia_lookback_window=21, delay = 0):
        super().__init__()
        self.starting_date = starting_date
        self.running_date = running_date
        self.calibration_starting_date = calibration_starting_date
        self.calibration_running_date = calibration_running_date

        self.freqly_return_pkl_filename_suffix=freqly_return_pkl_filename_suffix
        self.aggregated_pkl_file_suffix=aggregated_pkl_file_suffix
        self.aggregated_pkl_mapping_file_suffix=aggregated_pkl_mapping_file_suffix
        self.underlying = underlying
        self.list_pkl_file_suffix = list_pkl_file_suffix
        self.frequence=frequence
        self.algo=algo
        self.selected_algo=selected_algo
        self.delay = delay

        self.calibrating_dates_stub = self.calibration_starting_date.strftime('%d_%b_%Y') + '_' + self.calibration_running_date.strftime('%d_%b_%Y')
        self.dates_stub = self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y')

        if self.frequence == 'daily':
            self.list_pkl_file_name = self.calibrating_dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.list_pkl_file_suffix
            self.aggregated_pkl_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + '_' + str(
                self.delay) + self.aggregated_pkl_file_suffix
            self.aggregated_pkl_mapping_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + '_' + str(
                self.delay) + self.aggregated_pkl_mapping_file_suffix
        else:
            self.list_pkl_file_name = self.calibrating_dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.list_pkl_file_suffix
            self.aggregated_pkl_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.aggregated_pkl_file_suffix
            self.aggregated_pkl_mapping_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.aggregated_pkl_mapping_file_suffix

        self.local_root_directory=local_root_directory
        self.user=user
        self.dropbox_backup = dropbox_backup
        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
        self.running_date = running_date
        self.starting_date = starting_date
        self.add_turn_over = add_turn_over
        self.skipping_size = skipping_size
        self.signals_list = napoleon_connector.load_pickled_list(local_root_directory=self.local_root_directory,
                                                                 list_pkl_file_name=self.list_pkl_file_name)

        self.signals_list = list(self.signals_list)
        self.signals_list.sort()

        self.dates_stub = starting_date.strftime('%d_%b_%Y') + '_' + running_date.strftime('%d_%b_%Y')

        if self.frequence == 'daily':
            if self.delay == 0:
                self.saving_return_path = self.local_root_directory+self.underlying+ '_' + self.dates_stub + self.freqly_return_pkl_filename_suffix
            else :
                self.saving_return_path = f'{self.local_root_directory}{self.underlying}_{self.dates_stub}_{self.delay}{self.freqly_return_pkl_filename_suffix}'
        else :
            self.saving_return_path = self.local_root_directory+self.underlying+ '_' + self.dates_stub + self.freqly_return_pkl_filename_suffix


        self.recompute_all = recompute_all
        self.db_path_suffix = db_path_suffix
        if self.frequence == 'daily':
            self.db_path_name = launcher_utility.get_daily_database_name(user, frequence, underlying, algo, starting_date, running_date, delay, db_path_suffix)
        else:
            self.db_path_name = launcher_utility.get_database_name(user, frequence, underlying, algo, starting_date, running_date, db_path_suffix)

        # if not self.recompute_all:
        #     saver = signal_result_analyzer.ParallelRunResultAnalyzer(drop_token=drop_token,
        #                                                              local_root_directory=local_root_directory, user=user,
        #                                                              db_path_name=self.db_path_name)
        #     self.precomputed_results_df, self.precomputed_signals_df = saver.analyzeAllRunResults()
        # else:
        #     self.precomputed_results_df, self.precomputed_signals_df  = None, None
        print('prereading data from '+self.db_path_name )
        saver = signal_result_analyzer.ParallelRunResultAnalyzer(drop_token=drop_token,
                                                                 local_root_directory=local_root_directory, user=user,
                                                                 db_path_name=self.db_path_name)
        self.precomputed_results_df, self.precomputed_signals_df = saver.analyzeAllRunResults()
        self.daily_ia_lookback_window = daily_ia_lookback_window


        print('done')

    @abstractmethod
    def runTrial(self,saver):
        pass

class AggregatingSignalRunner(AbstractRunner):
    def backup_to_dropbox(self):
        local_root_directory = Path(self.local_root_directory)
        aggregated_csv_file_name = self.aggregated_pkl_file_name.replace('pkl', 'csv')
        aggregated_csv_mapping_file_name = self.aggregated_pkl_mapping_file_name.replace('pkl', 'csv')

        aggregated_df = pd.read_pickle(local_root_directory / self.aggregated_pkl_file_name)
        aggregated_mapping_df = pd.read_pickle(local_root_directory / self.aggregated_pkl_mapping_file_name)

        aggregated_csv_full_path = local_root_directory / aggregated_csv_file_name
        aggregated_csv_mapping_full_path = local_root_directory / aggregated_csv_mapping_file_name

        aggregated_df.to_csv(aggregated_csv_full_path)
        aggregated_mapping_df.to_csv(aggregated_csv_mapping_full_path)
        print('backuping to dropbox '+str(aggregated_csv_full_path))
        self.dbx.upload(fullname=aggregated_csv_full_path, folder='', subfolder='', name=aggregated_csv_file_name,overwrite=True)
        print('backuping to dropbox '+str(aggregated_csv_mapping_full_path))
        self.dbx.upload(fullname=aggregated_csv_mapping_full_path, folder='', subfolder='',name=aggregated_csv_mapping_file_name, overwrite=True)


    def runSingleTrial(self, me_signal):
        try:
            print('Launching ' + me_signal)
            # we have to reload the file each time
            freqly_df = pd.read_pickle(self.saving_return_path)
            freqly_df = freqly_df.sort_index()
            print('time range before filtering ')
            print(max(freqly_df.index))
            print(min(freqly_df.index))
            freqly_df = freqly_df[freqly_df.index >= self.starting_date]
            freqly_df = freqly_df[freqly_df.index <= self.running_date]
            print('time range after filtering ')
            print(max(freqly_df.index))
            print(min(freqly_df.index))

            ## idiosyncratic run itself
            run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
            params = json.loads(run_json_string)
            signal_type = params['signal_type']
            normalization = params['normalization']
            trigger = params['trigger']
            transaction_costs = params['transaction_costs']

            if normalization and not signal_generator.is_signal_continuum(signal_type):
                return

            print('Launching computation with parameters : ' + run_json_string)

            if signal_type == 'long_only':
                freqly_df['signal'] = 1.
            else:
                lookback_window = params['lookback_window']
                if self.skipping_size < lookback_window:
                    raise Exception('The skipping size must be greater than the lookback window')
                # kwargs = {**generics, **idios}
                signal_generation_method_to_call = getattr(signal_generator, signal_type)
                freqly_df = signal_utility.roll_wrapper(freqly_df, lookback_window, self.skipping_size,
                                                        lambda x: signal_generation_method_to_call(data=x, **params),
                                                        trigger)
                freqly_df, _ = signal_utility.reconstitute_signal_perf(data=freqly_df,
                                                                       transaction_cost=transaction_costs,
                                                                       normalization=normalization)

            if self.add_turn_over:
                cumulated_signals = freqly_df[['close', 'signal', 'turn_over']].copy()
                cumulated_signals = cumulated_signals.rename(
                    columns={'signal': me_signal, 'turn_over': 'turn_over' + me_signal})
            else:
                cumulated_signals = freqly_df[['close', 'signal']].copy()
                cumulated_signals = cumulated_signals.rename(columns={'signal': me_signal})


            print('diiscretizing the signal ')
            return cumulated_signals
        except Exception as e:
            print('cannot compute ', me_signal)
            print(e)
            return None


    def runTrial(self, saver):
        cumulated_signals = None
        print('Number of signals to aggregate '+str(len(self.signals_list)))
        counter = 1
        mapping_dictionary = {}
        for me_signal in self.signals_list:
            # we have to reload the file each time
            freqly_df = pd.read_pickle(self.saving_return_path)
            freqly_df = freqly_df.sort_index()
            print('time range before filtering ')
            print(max(freqly_df.index))
            print(min(freqly_df.index))
            freqly_df = freqly_df[freqly_df.index >= self.starting_date]
            freqly_df = freqly_df[freqly_df.index <= self.running_date]
            print('time range after filtering ')
            print(max(freqly_df.index))
            print(min(freqly_df.index))
            close_df = freqly_df.copy()

            ## idiosyncratic run itself
            run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
            params = json.loads(run_json_string)
            signal_type = params['signal_type']
            normalization = params['normalization']
            trigger = params['trigger']
            transaction_costs = params['transaction_costs']

            if normalization and not signal_generator.is_signal_continuum(signal_type):
                return
            signal_column = 'signal'+str(counter)
            mapping_dictionary[me_signal]=[signal_column]
            if self.recompute_all:
                print('Launching computation with parameters : '+run_json_string)
                print('Signal number '+str(counter))
                # try:
                #     testing_presence_df = self.precomputed_results_df[me_signal]
                # except KeyError:
                #     print('cannot read ', me_signal)
                #     continue

                if signal_type == 'long_only':
                    freqly_df['signal']=1.
                else:
                    lookback_window = params['lookback_window']
                    if self.skipping_size < lookback_window:
                        raise Exception('The skipping size must be greater than the lookback window')
                    # kwargs = {**generics, **idios}
                    signal_generation_method_to_call = getattr(signal_generator, signal_type)
                    freqly_df = signal_utility.roll_wrapper(freqly_df, lookback_window, self.skipping_size,
                                              lambda x: signal_generation_method_to_call(data=x, **params), trigger)
                    freqly_df, _ = signal_utility.reconstitute_signal_perf(data = freqly_df, transaction_cost = transaction_costs, normalization = normalization)
            else :
                if self.add_turn_over:
                    raise Exception('to add turn over, you have to recompute all')
                try:
                    freqlyperf_df = self.precomputed_results_df[me_signal]
                except KeyError:
                    print('cannot read ', me_signal)
                    continue
                try:
                    freqlysig_df = self.precomputed_signals_df[me_signal]
                except KeyError:
                    print('cannot read ', me_signal)
                    continue
                freqlyperf_df = freqlyperf_df.to_frame()
                freqlyperf_df.columns = ['reconstituted_perf']
                freqlysig_df = freqlysig_df.to_frame()
                freqlysig_df.columns = ['signal']
                freqly_df = freqlyperf_df.merge(freqlysig_df, left_index = True, right_index = True)
                if cumulated_signals is None :
                    freqlyclose_df = close_df['close'].to_frame()
                    freqlyclose_df.columns = ['close']
                    freqly_df = freqly_df.merge(freqlyclose_df, left_index=True, right_index=True)
            if cumulated_signals is None:
                if self.add_turn_over:
                    cumulated_signals = freqly_df[['close','signal','turn_over']]
                    cumulated_signals = cumulated_signals.rename(columns={'signal': signal_column,'turn_over':'turn_over'+str(counter)})
                else :
                    cumulated_signals = freqly_df[['close','signal']]
                    cumulated_signals = cumulated_signals.rename(columns={'signal': signal_column})
            else :
                cumulated_signals[signal_column]=freqly_df['signal']
                if self.add_turn_over:
                    cumulated_signals['turn_over'+str(counter)]=freqly_df['turn_over']
            #freqly_df = freqly_df.drop(columns = ['signal','turn_over','reconstituted_perf','non_adjusted_perf_return','perf_return'])
            counter = counter + 1


        self.saving_cumulated_signals(cumulated_signals,mapping_dictionary)


    def saving_cumulated_signals(self, cumulated_signals_df, mapping_dictionary):
        print('discretizing the cumulated signals using ia')
        discrete_daily_signals = daily_bc_apply_standard_mean_discretization(signals_df=cumulated_signals_df, s=self.daily_ia_lookback_window)
        cumulated_signals_df['trend']=discrete_daily_signals
        print(f'discretization done {len(discrete_daily_signals)}')
        print('saving to ' + self.aggregated_pkl_file_name)
        cumulated_signals_df.to_pickle(self.local_root_directory + self.aggregated_pkl_file_name)
        print('saving to '+self.aggregated_pkl_mapping_file_name)
        pd.DataFrame(mapping_dictionary).to_pickle(self.local_root_directory + self.aggregated_pkl_mapping_file_name)







